/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctOption.h
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

#ifndef CT_OPTION_H
#define CT_OPTION_H

/*------------------------------------------------------------- Includes */

#include <qvbox.h>
#include <qstring.h>

/*------------------------------------------- Forward class declarations */

class QPushButton;
class QLineEdit;
#ifdef QTVER2X
class QMultiLineEdit;
#else
class QTextEdit;
#endif
class QCheckBox;
class QListBox;
class QGroupBox;
class ctDrawBoard;
class ctTaskList;
class ctProgressDialog;

/*-------------------------------------------------------------- Classes */

/** This class represents the option widget in the
application. It is the central point of the program,
managing the list of tasks \a tlTasks and updating
the chess board \a dbDrawingBoard, if necessary.
*/
class ctOption : public QVBox
{
    Q_OBJECT

public:
    ctOption( QWidget *parent, const char *name = 0 );
    ~ctOption();

public slots:
    void slotOpenFile();
    void slotAddFile();
    void slotSaveFile();
    void slotExportLaTeXFile();
    void slotExportChessCardsFile();
    void slotExportHTML();
    void slotExportLaTeXDirectory();
    void slotExportChessCardsDirectory();
    void slotExportHTMLDirectory();
    void slotCopyLaTeXPosition();
    void slotCopyHTMLPosition();
    void slotImportFENFile();
    void slotImportEPDFile();
    void slotDeleteAll();

    void slotInsertTask();
    void slotCopyTask();
    void slotDeleteTask();
    void slotTaskBack();
    void slotTaskForward();
    void slotFirstTask();
    void slotLastTask();

    void slotRestoreTask();
    void slotSaveTask();
    
    void slotSetMainTitle();
    void slotGenerateTitles();
    void slotSetDirLevel(int);

protected:

protected slots:
    void slotPieceChangedToPawn();
    void slotPieceChangedToKnight();
    void slotPieceChangedToBishop();
    void slotPieceChangedToRook();
    void slotPieceChangedToQueen();
    void slotPieceChangedToKing();
    void slotPieceChangedToMarkerA();
    void slotPieceChangedToMarkerB();
    void slotColorChangedToWhite();
    void slotColorChangedToBlack();

    void slotInsertBoldFaceCommand();
    void slotInsertItalicFaceCommand();
    void slotInsertMoveCommand();
    void slotInsertLaTeXHTMLCommand();
    void slotInsertInformatorCommand();
    void slotInsertKnightMoveCommand();
    void slotInsertArrowCommand();
    void slotInsertHighlightCommand();


private:
    /** The drawing board */
    ctDrawBoard *dbDrawingBoard;
    /** The list of tasks */
    ctTaskList *tlTasks;

    /** Button for setting the color to black */
    QPushButton *pbSetBlack;
    /** Button for setting the color to white */
    QPushButton *pbSetWhite;

    /** Button for setting the piece to pawn */
    QPushButton *pbSetPawn;
    /** Button for setting the piece to knight */
    QPushButton *pbSetKnight;
    /** Button for setting the piece to bishop */
    QPushButton *pbSetBishop;
    /** Button for setting the piece to rook */
    QPushButton *pbSetRook;
    /** Button for setting the piece to queen */
    QPushButton *pbSetQueen;
    /** Button for setting the piece to king */
    QPushButton *pbSetKing;
    /** Button for setting the piece to Marker A */
    QPushButton *pbSetMarkerA;
    /** Button for setting the piece to Marker B */
    QPushButton *pbSetMarkerB;

    /** Button for displaying the selected piece */
    QPushButton *pbSelectedPiece;

    /** Button for inserting a bold face command into the current line edit */
    QPushButton *pbBoldFaceCommand;
    /** Button for inserting an italic face command into the current line edit */
    QPushButton *pbItalicFaceCommand;
    /** Button for inserting a variation command into the current line edit */
    QPushButton *pbMoveCommand;
    /** Button for inserting an informator command into the current line edit */
    QPushButton *pbInformatorCommand;

    /** Check box for enabling the printing of the diagram */
    QCheckBox *cbDiagram;
    /** Check box for trailing space after LaTeX commands, i.e. informator symbols */
    QCheckBox *cbTrailingSpace;

    /** List box for Informator symbols */
    QListBox *lbInformator;

    /** Line edit for the title of the task */
    QLineEdit *leTitle;

#ifdef QTVER2X
    /** Text multiline edit for the task text */
    QMultiLineEdit *mleTask;
    /** Text multiline edit for the solution text */
    QMultiLineEdit *mleSolution;
#else
    /** Text multiline edit for the task text */
    QTextEdit *mleTask;
    /** Text multiline edit for the solution text */
    QTextEdit *mleSolution;
#endif

    /** Currently selected color */
    unsigned int currentColor;
    /** Currently selected piece */
    unsigned int currentPiece;

    /** The relative path to the HTML ``graphics'' directory */
    QString graphicsDir;

    /** The dialog for displaying the ``export'' progress */
    ctProgressDialog *pdProgress;
    /** List of tasks for the ``export'' to HTML or LaTeX, respectively */
    ctTaskList *tlExport;

    void setupOptions();
    void setupConnects();

    int getActiveLineEdit();
    int getNumberOfTaskFiles(const QString &searchDir);
    void recursiveLaTeXDirectoryExport(int exportType,
                                       const QString &sourceDirName,
                                       const QString &destDirName,
				       int &fileNumber);
    void recursiveHTMLDirectoryExport(const QString &sourceDirName,
                                      const QString &destDirName,
				      QStringList &slIndex,
				      int &fileNumber);
    void directoryExport(int exportType);

};

#endif


