/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctPosition.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctPosition.h"
#include "ctPieces.h"

#include <qregexp.h>

/*------------------------------------------------------------ Functions */

/** Default constructor for the \a ctPosition class.
*/
ctPosition::ctPosition() : 
        QObject()
{
  int i;

  // Initialize to empty board
  for (i = 0; i < 64; i++)
    Position[i] = EMPTY;

}

/** Copy constructor for the \a ctPosition class.
@param elem Reference to the position that is copied
*/
ctPosition::ctPosition(const ctPosition &elem) : 
        QObject()
{
  int i;

  // Copy the board
  for (i = 0; i < 64; i++)
    Position[i] = elem.Position[i];

}

/** The overloaded operator ``='' with another \a ctPosition 
as RHA.
@param elem Reference to the position that is copied
@return Reference to this \a ctPosition
*/
ctPosition &ctPosition::operator= (const ctPosition &elem)
{

  if (this == &elem)
    return *this;
  
  // Counter
  int i;

  for (i = 0; i < 64; i++)
    Position[i] = elem.Position[i];

  return *this;
}

/** Imports a position in FEN notation. 
Scans the given diagram string \a fenString and 
derives the position of the pieces from it. If the string
does not have the correct format, the currently stored
position remains unchanged.
@param fenString Reference to the string
*/
void ctPosition::importFEN(const QString &fenString)
{
  // Local copy of the string
  QString diagram = fenString;

  // Remove slashes
  diagram.replace(QRegExp("/"), "");
  // Replace empty squares
  diagram.replace(QRegExp("1"), " ");
  diagram.replace(QRegExp("2"), "  ");
  diagram.replace(QRegExp("3"), "   ");
  diagram.replace(QRegExp("4"), "    ");
  diagram.replace(QRegExp("5"), "     ");
  diagram.replace(QRegExp("6"), "      ");
  diagram.replace(QRegExp("7"), "       ");
  diagram.replace(QRegExp("8"), "        ");

  // Has the string the correct length?
  if (diagram.length() == 64)
  {
    // Yes, so set up the diagram

    // Counter
    int i = 0;

    for (; i < 64; i++)
    {
      switch (diagram[i].unicode() & 255)
      {
        case 'P': Position[i] = PAWN; break;
        case 'N': Position[i] = KNIGHT; break;
        case 'B': Position[i] = BISHOP; break;
        case 'R': Position[i] = ROOK; break;
        case 'Q': Position[i] = QUEEN; break;
        case 'K': Position[i] = KING; break;
        case 'p': Position[i] = BPAWN; break;
        case 'n': Position[i] = BKNIGHT; break;
        case 'b': Position[i] = BBISHOP; break;
        case 'r': Position[i] = BROOK; break;
        case 'q': Position[i] = BQUEEN; break;
        case 'k': Position[i] = BKING; break;
        default : Position[i] = EMPTY; break;
      }
    }
  }

}

/** Reads a line of text (ended by a ``Newline'') 
from the QTextStream \a t and tries to interpret
it as a position in the ``ChessTask'' format.
@param t Reference to the text stream
*/
void ctPosition::load(QTextStream &t)
{
  // Read the next line
  QString input = t.readLine();
  // Counter
  int i = 0;
  // Position of the next comma
  int comma = 0;

  while ((!input.isEmpty()) && (i < 63))
  {
    // Find next comma
    comma = input.find(",");

    // Convert string to int
    Position[i] = input.mid(0, comma).toInt();

    // Remove first number from string
    input.remove(0, comma+1);

    i++;
  }  
    
  // Convert last number
  if (!input.isEmpty())
  {
    Position[63] = input.toInt();
  }  
}  

/** Writes the position into the QTextStream \a t.
A single line of text is generated, containing the
IDs of the single pieces, separated by commas.
@param t Reference to the text stream
*/
void ctPosition::save(QTextStream &t)
{
  // Counter
  int i;

  for (i = 0; i < 63; i++)
  {
    t << Position[i] << ",";
  }
  t << Position[i];

}

/** Overloaded ``matrix access'' operator. The variables
\a row and \a col refer to the position on the chess board.
The valid range for both is 0--7.
@param row The row
@param col The column
@return Reference to the chess piece at the specified position
*/
unsigned int &ctPosition::operator() (unsigned row, unsigned col)
{
  return Position[row*8+col];
}

/** Overloaded ``matrix access'' operator. The variables
\a row and \a col refer to the position on the chess board.
The valid range for both is 0--7.
@param row The row
@param col The column
@return ID of the chess piece at the specified position
*/
unsigned int ctPosition::operator() (unsigned row, unsigned col) const
{
  return Position[row*8+col];
}

/** Outputs the letter for the given line \a iLine to the
QTextStream \a t.
@param t Reference to the QTextStream
@param iLine Index of the line (0--7)
*/
void ctPosition::writeLine(QTextStream &t, int iLine)
{
  switch (iLine)
  {
    case 0: t << "a";
            break;
    case 1: t << "b";
            break;
    case 2: t << "c";
            break;
    case 3: t << "d";
            break;
    case 4: t << "e";
            break;
    case 5: t << "f";
            break;
    case 6: t << "g";
            break;
    case 7: t << "h";
            break;
  } 
}


/** Outputs the chess board in FEN notation to the
QTextStream \a t.
@param t Reference to the QTextStream
*/
void ctPosition::exportFEN(QTextStream &t)
{
  // The current piece
  unsigned int currentPiece;
  // Counters
  int i, j;
  // The number of consecutive empty squares
  int emptySquares;

  for (i = 0; i < 8; i++)
  {
    // Write a single line
    j = 0;
    while (j < 8)
    {
      currentPiece = Position[i*8 + j];
      // Is the current piece not empty and not a marker?
      if ((currentPiece != EMPTY) && (currentPiece != MARKER_A) &&
          (currentPiece != MARKER_B) && (currentPiece != BMARKER_A) &&
          (currentPiece != BMARKER_B))
      {
        // Yes, so output the corresponding letter
        switch (currentPiece)
        {
          case PAWN: t << "P"; break;
          case KNIGHT: t << "N"; break;
          case BISHOP: t << "B"; break;
          case ROOK: t << "R"; break;
          case QUEEN: t << "Q"; break;
          case KING: t << "K"; break;
          case BPAWN: t << "p"; break;
          case BKNIGHT: t << "n"; break;
          case BBISHOP: t << "b"; break;
          case BROOK: t << "r"; break;
          case BQUEEN: t << "q"; break;
          case BKING: t << "k"; break;
          default: t << tr("e"); break;
        }
        j++;
      }
      else
      {
        emptySquares = 0;
        // No, so ``pack'' empty squares for FEN
        while ((j < 8) && 
               ((currentPiece == EMPTY) || (currentPiece == MARKER_A) ||
                (currentPiece == MARKER_B) || (currentPiece == BMARKER_A) ||
                (currentPiece == BMARKER_B)))
        {
          j++;
          emptySquares++;
          if (j < 8)
            currentPiece = Position[i*8 + j];
        }
        // Output number of empty squares
        t << emptySquares;
      }
    }
    // Is it not the last line?
    if (i < 7)
      t << "/";
  }
  // These are ``dummy values'' as default...we only want to display
  // the board instead of playing the game any further.
  t << " w KQkq - 0 1";

}

/** Outputs appropriate TeX lines for the marker
symbols to the QTextStream \a t.
@param t Reference to the QTextStream
*/
void ctPosition::exportMarkerLines(QTextStream &t)
{
  // The current piece
  unsigned int currentPiece;
  // Counters
  int i, j;
  // Do we have to write a line for the markers?
  bool bMarkerA = false, bMarkerB = false;

  // Check if there are any markers...
  for (i = 0; i < 8; i++)
  {
    for (j = 0; j < 8; j++)
    {
      currentPiece = Position[i*8 + j];

      if ((currentPiece == MARKER_A) || (currentPiece == BMARKER_A))
        bMarkerA = true;
      if ((currentPiece == MARKER_B) || (currentPiece == BMARKER_B))
        bMarkerB = true;
    }
  }

  // Do we have to write a MarkerA line?
  if (bMarkerA == true)
  {
    t << "\\highlight[x]{";
    for (i = 0; i < 8; i++)
    {
      for (j = 0; j < 8; j++)
      {
        currentPiece = Position[i*8 + j];
        if ((currentPiece == MARKER_A) || (currentPiece == BMARKER_A))
        {
          // Is this the first marker that we write?
          if (bMarkerA == true)
          {
            // Yes, so reset our ``flag'' to false...
            bMarkerA = false;
          }
          else
          {
            // No, so prepend a comma
            t << ",";
          }
          // Output coordinates
          writeLine(t, j);
          t << (8-i);
        }
      }
    }
    t << "}" << endl;
  }

  // Do we have to write a MarkerB line?
  if (bMarkerB == true)
  {
    t << "\\highlight[o]{";
    for (i = 0; i < 8; i++)
    {
      for (j = 0; j < 8; j++)
      {
        currentPiece = Position[i*8 + j];
        if ((currentPiece == MARKER_B) || (currentPiece == BMARKER_B))
        {
          // Is this the first marker that we write?
          if (bMarkerB == true)
          {
            // Yes, so reset our ``flag'' to false...
            bMarkerB = false;
          }
          else
          {
            // No, so prepend a comma
            t << ",";
          }
          // Output coordinates
          writeLine(t, j);
          t << (8-i);
        }
      }
    }
    t << "}" << endl;
  }

}

/** Outputs the chess board diagram in LaTeX format to the
QTextStream \a t.
@param t Reference to the QTextStream
*/
void ctPosition::exportLaTeX(QTextStream &t)
{
  // Write position
  t << "\\fenboard{";
  exportFEN(t);
  t << "}" << endl;

  // Show the board
  t << "\\centerline{\\showboard}" << endl;
	  
  // Write marker lines
  exportMarkerLines(t);

}


/** Outputs the chess board diagram in HTML format to the
QTextStream \a t.
@param t Reference to the QTextStream
@param graphicsDir Reference to the string containing the ``graphics''
directory
*/
void ctPosition::exportHTML(QTextStream &t, const QString &graphicsDir)
{
  // Counters
  int i, j;

  t << "<CENTER>" << endl;
  // Main table
  t << "<TABLE BORBER=0 CELLPADDING=0 CELLSPACING=0>" << endl;
  t << "<TR>\n<TD>" << endl;
  // Outer table of the board, responsible for the border frame
  t << "<TABLE BORDERCOLOR=BLACK BORDER=2 CELLPADDING=0 CELLSPACING=0>" << endl;
  t << "<TR>\n<TD>" << endl;
  // Inner table of the board, containing the pieces
  t << "<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0>" << endl;

  // Write rows
  for (i = 0; i < 8; i++)
  {
    t << "<TR>" << endl;
    for (j = 0; j < 8; j++)
    {
      t << "<TD><IMG SRC=\"" << graphicsDir;

      // Is the field white or black?
      if (((i*9 + j) % 2) == 0)
      {
	// White
	t << "wf";
      }  
      else 
      {
        // Black
        t << "sf";
      }

      switch (Position[i*8 + j])
      {
        case PAWN: t << "wb"; break;
        case KNIGHT: t << "ws"; break;
        case BISHOP: t << "wl"; break;
        case ROOK: t << "wt"; break;
        case QUEEN: t << "wd"; break;
        case KING: t << "wk"; break;
        case MARKER_A: t << "ma"; break;
        case MARKER_B: t << "mb"; break;
        case BPAWN: t << "sb"; break;
        case BKNIGHT: t << "ss"; break;
        case BBISHOP: t << "sl"; break;
        case BROOK: t << "st"; break;
        case BQUEEN: t << "sd"; break;
        case BKING: t << "sk"; break;
        case BMARKER_A: t << "ma"; break;
        case BMARKER_B: t << "mb"; break;
        default: t << ""; break;
      }  
      t << ".png\"></TD>" << endl;
    }
    t << "</TR>" << endl;
  }

  // Closing the inner table of the board
  t << "</TABLE>" << endl;
  t << "</TD>\n</TR>" << endl;

  // Closing the outer table of the board
  t << "</TABLE>\n</TD>" << endl;

  // Writing the notation letters for the rows
  t << "<TD>\n<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0>" << endl;
  for (i = 8; i > 0; i--)
  {
    t << "<TR>" << endl;
    t << "<TD ALIGN=CENTER VALIGN=CENTER WIDTH=29 HEIGHT=29>";
    t << i << "</TD>" << endl;
    t << "</TR>" << endl;
  }
  // Closing the table for the rows notation
  t << "</TABLE>\n</TD>\n</TR>" << endl;

  // Second row of main table: The notation letters for the columns
  t << "<TR>\n<TD>\n<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0>" << endl;
  t << "<TR>" << endl;

  for (i = 0; i < 8; i++)
  {
    t << "<TD ALIGN=CENTER VALIGN=CENTER WIDTH=29 HEIGHT=29>";
    t << QString((QChar) (65+i)) << "</TD>" << endl;
  }
  
  t << "</TR>\n</TABLE>\n</TD>" << endl;

  // ``Dummy'' table entry, and end of table
  t << "<TD></TD>\n</TR>\n</TABLE>" << endl;

  t << "</CENTER>" << endl;

}


/** Clears the board.
*/
void ctPosition::clearBoard()
{
  int i;

  // Initialize to empty board
  for (i = 0; i < 64; i++)
    Position[i] = EMPTY;

}

/** Sets the board to the initial position.
*/
void ctPosition::initBoard()
{
  int i;

  // Set black pieces
  Position[0] = BROOK;
  Position[1] = BKNIGHT;
  Position[2] = BBISHOP;
  Position[3] = BQUEEN;
  Position[4] = BKING;
  Position[5] = BBISHOP;
  Position[6] = BKNIGHT;
  Position[7] = BROOK;
  for (i = 8; i < 16; i++)
    Position[i] = BPAWN;

  for (i = 16; i < 48; i++)
    Position[i] = EMPTY;

  // Set white pieces
  for (i = 48; i < 56; i++)
    Position[i] = PAWN;
  Position[56] = ROOK;
  Position[57] = KNIGHT;
  Position[58] = BISHOP;
  Position[59] = QUEEN;
  Position[60] = KING;
  Position[61] = BISHOP;
  Position[62] = KNIGHT;
  Position[63] = ROOK;

}

