/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctString.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctString.h"
#include "ctPieces.h"
#include "ctGlobal.h"

#include <qapplication.h>
#include <qregexp.h>

#include <vector>

using namespace std;

/*--------------------------------------------------------- Const values */

/** The value for the ``roman font face'' state */
const int ROMAN = 0;
/** The value for the ``bold font face'' state */
const int BOLD = 1;
/** The value for the ``italic font face'' state */
const int ITALIC = 2;
/** The value for the ``bold and italic font face'' state */
const int BOLD_ITALIC = 3;
/** The value for the ``emphasize'' command */
const int EMPHASIZE = 4;
/** The value for the closing brace */
const int CLOSING_BRACE = 5;

/*------------------------------------------------------------ Functions */

/** The ``special'' constructor for the \a ctString class. The
string \a elem is exported to LATEX or HTML format, depending on
the value of \a iType. If the \a iType is CONVERT the string is
converted from the old file format (< v2.x) to the new one.
@param elem The string
@param iType The format the string is converted to (LATEX, HTML or CONVERT)
@param graphicsDir Path to the ``graphics'' directory
@param bExpand Is ``true'' if the Informator symbols should be output
in plain text, ``false'' if the \c ALT parameter should be used.
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
*/
ctString::ctString(const QString &elem, 
                   int iType,
		   const QString &graphicsDir,
		   bool bExpand,
                   bool bUseTTFont) : 
        QString(elem)
{
  if (iType == LATEX)
  {
    convertToLaTeX();
  }
  if (iType == HTML)
  {
    convertToHTML(graphicsDir, bExpand, bUseTTFont);
  }  
  if (iType == CONVERT)
  {
    convertFromOldFormat();
  }
}


/** Scans the string from the starting position
\a iStartPos to find a matching brace. The character at
the start has to be the ``opening'' brace. 
@param iStartPos Starting position in the string, has to be an
opening bracket
@return The position of the matching brace if it could be found, -1 else
*/
int ctString::findMatchingBrace(int iStartPos)
{
  if (at(iStartPos) != '{')
  {
    return(-1);
  }  
    
  // Number of unmatched brackets
  int iUnmatchedBrackets = 1;
  // The regular expression we are looking for
#ifdef QTVER2X
  QRegExp reBrackets("[{}]");
#else
  QRegExp reBrackets("[\\{\\}]");
#endif

  // Scan string
  while ((iUnmatchedBrackets > 0) && (iStartPos >= 0))
  {
    // Try to find the next brace
    iStartPos = find(reBrackets, iStartPos + 1);
    // Did we find a bracket?
    if (iStartPos > 0)
    {
      // Yes...is it an ``opening'' brace?
      if (at(iStartPos) == '{')
      {
        iUnmatchedBrackets++;
      }
      else
      {
        iUnmatchedBrackets--;
      }
    }  
  }

  return(iStartPos);
}  

/** Converts the \a ctString to LaTeX format. 
The TeX input is parsed for the command
\c \\latexhtml{}{} which is substituted by its first argument.
*/
void ctString::convertToLaTeX()
{
  // Index
  int idx, bidx;

  // Search for ``latexhtml'' command
  idx = find(QRegExp("\\\\latexhtml"));
  while (idx >=0)
  {
    // Remove ``latexhtml'' part
    remove(idx, 10);
    // Look for matching closing bracket
    bidx = findMatchingBrace(idx);
    
    if (bidx > 0)
    {
      // Cut out the closing bracket
      remove(bidx, 1);
      // Cut out the opening bracket
      remove(idx, 1);
      // Correct position counter
      idx = bidx - 1;

      // Look for matching closing bracket
      bidx = findMatchingBrace(idx);

      // Cut out the argument
      if (bidx > 0)
      {
        //... up to the matching bracket
        remove(idx, (bidx-idx+1));
      }
      else
      {
        //... up to the end of the string
        truncate(idx);
        return;
      }  
    
    }
    else
    {
      // No matching bracket found ->
      // accept everything up to the rest of the string

      return;
    }

    // Try to find another ``latexhtml'' command
    if (idx > 0) 
      idx--;
    idx = find(QRegExp("\\\\latexhtml"), idx);
  }

  // If the resulting string is empty...
  if (isEmpty() == true)
  {
    // ... a single blank is inserted for the output.
    // This should prevent empty task and solution texts
    // from confusing the TeX/LaTeX run, especially for
    // the ChessCards format.
    append("\\ ");
  }
}

/** Converts the string to HTML format. The TeX input is parsed
for commands and ``bold'' or ``italic'' environments. These, 
like ``move environments'', are substituted by the equivalent
HTML commands.
@param graphicsDir The graphics directory
@param bExpand Is ``true'' if the Informator symbols should be output
in plain text, ``false'' if the \c ALT parameter should be used.
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
*/
void ctString::convertToHTML(const QString &graphicsDir, 
                             bool bExpand, bool bUseTTFont)
{
  // Some positions for scanning through the string
  int idx, bidx;

  // Search for ``latexhtml'' command
  idx = find(QRegExp("\\\\latexhtml"));
  while (idx >=0)
  {
    // Remove ``latexhtml'' part
    remove(idx, 10);
    // Look for matching closing bracket
    bidx = findMatchingBrace(idx);
    
    // Cut out the argument
    if (bidx > 0)
    {
      //... up to the matching bracket
      remove(idx, (bidx-idx+1));
    }
    else
    {
      //... up to the end of the string
      truncate(idx);

      // Leave the ``while'' loop
      break;
    }  

    // Look for matching closing bracket
    bidx = findMatchingBrace(idx);

    if (bidx > 0)
    {
      // Cut out the matching bracket
      remove(bidx, 1);
    }
      
    // Cut out the first bracket
    remove(idx, 1);

    // Try to find another ``latexhtml'' command
    if (idx > 0)
      idx--;
    idx = find(QRegExp("\\\\latexhtml"), idx);
  }

  // Replace start of new paragraphs
  replace(QRegExp("\\n\\n+"), "\n<P>\n");

  // Replace inter-word space commands
  replace(QRegExp("\\\\-"), "");
  replace(QRegExp("\\\\/"), "");
  replace(QRegExp("\\\\,"), "");
  replace(QRegExp("\\\\@"), "");
  replace(QRegExp("\\\\ "), " ");
  replace(QRegExp("\\\\_"), "_");
  replace(QRegExp("~"), "&nbsp;");

  // Replace informator symbols
  processInformatorSymbols(graphicsDir, bExpand, bUseTTFont);

  // Remarks: So far, not all informator symbols have
  // been processed...
  // The symbols ``comment'', ``various'' and ``novelty''
  // are converted twice. First, while replacing ``movecomment''
  // environments, they have to be ``escaped'' by the tags
  // </SPAN>N<SPAN> because, as an example, the `N' in the 
  // ``DBChess'' font is reserved for the knight symbol.
  // Afterwards, the commands outside of any ``movecomment'' envs
  // can be replaced by the simple letter combinations 
  // ``N'', ``R'' and ``RR''.

  // Process ``movecomment'' environments
  processMoveEnvironments(bUseTTFont);

  // Replace commands without <SPAN> tags
  // Replace ``novelty'' command
  replacePlainSymbol("novelty", "N", false, "N");
  // Replace ``various'' command
  replacePlainSymbol("various", "R", false, "R");
  // Replace ``comment'' command
  replacePlainSymbol("comment", "RR", false, "RR");

  // Final step: Replacing text environments
  processTextEnvironments();

}

/** Converts the string from the old file formats (< v2.x) to
the new one. Replaces ``:'' and ``*'' within the old ``move''
environment ``||'' by ``...,'' and ``x'', respectively.
The ``move'' environment itself is replaced by ``\movecomment{}''.
*/
void ctString::convertFromOldFormat()
{
  // Some positions for scanning through the string
  int idx, bidx;

  // Process ``old move'' environments
  idx = find('|');
  while (idx >=0)
  {
    replace(idx, 1, "\\movecomment{");
    idx += 12;

    // Look for the closing delimiter
    bidx = find('|', idx);
    if (bidx > 0)
    {
      replace(bidx, 1, "}");

      // Find captures
      replaceMoveSymbol(idx, &bidx, QRegExp("\\*"), 
                        "x", false, "x", false);
      // Find colons
      replaceMoveSymbol(idx, &bidx, QRegExp(":"), 
                        " ...,", false, " ...,", false);
    }
    idx = find('|');
  } // End of: Replacing old ``move'' environments

} 

/** Replaces the informator symbols with their HTML equivalent.
@param graphicsDir The graphics directory
@param bExpand Is ``true'' if the Informator symbols should be output
in plain text, ``false'' if the \c ALT parameter should be used.
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
*/
void ctString::processInformatorSymbols(const QString &graphicsDir, 
                                        bool bExpand, 
                                        bool bUseTTFont)
{
  // Replace ``mate'' command
  replacePlainSymbol("mate", "#", false, "#");
  // Replace ``equal'' command
  replacePlainSymbol("equal", "=", false, "=");
  // Replace ``markera'' command
  replacePlainSymbol("markera", "<IMG SRC=\"" + graphicsDir + "wfma.png\" WIDTH=12 HEIGHT=12> ", bUseTTFont, "X");
  // Replace ``markerb'' command
  replacePlainSymbol("markerb", "<IMG SRC=\"" + graphicsDir + "wfmb.png\" WIDTH=12 HEIGHT=12> ", bUseTTFont, "O");

  // Replace rest of informator symbols
  replaceInformatorSymbol("wbetter", qApp->translate("ctString", "white stands slightly better"), graphicsDir, bExpand, bUseTTFont, "@");
  replaceInformatorSymbol("bbetter", qApp->translate("ctString", "black stands slightly better"), graphicsDir, bExpand, bUseTTFont, "A");
  replaceInformatorSymbol("wupperhand", qApp->translate("ctString", "white has the upper hand"), graphicsDir, bExpand, bUseTTFont, "C");
  replaceInformatorSymbol("bupperhand", qApp->translate("ctString", "black has the upper hand"), graphicsDir, bExpand, bUseTTFont, "E");
  replaceInformatorSymbol("wdecisive", qApp->translate("ctString", "white has a decisive advantage"), graphicsDir, bExpand, bUseTTFont, "+-");
  replaceInformatorSymbol("bdecisive", qApp->translate("ctString", "black has a decisive advantage"), graphicsDir, bExpand, bUseTTFont, "-+");
  replaceInformatorSymbol("unclear", qApp->translate("ctString", "unclear"), graphicsDir, bExpand, bUseTTFont, "F");
  replaceInformatorSymbol("compensation", qApp->translate("ctString", "with compensation for the material"), graphicsDir, bExpand, bUseTTFont, "G");
  replaceInformatorSymbol("devadvantage", qApp->translate("ctString", "development advantage"), graphicsDir, bExpand, bUseTTFont, "H");
  replaceInformatorSymbol("moreroom", qApp->translate("ctString", "greater board room"), graphicsDir, bExpand, bUseTTFont, "O");
  replaceInformatorSymbol("withattack", qApp->translate("ctString", "with attack"), graphicsDir, bExpand, bUseTTFont, "I");
  replaceInformatorSymbol("withinit", qApp->translate("ctString", "with initiative"), graphicsDir, bExpand, bUseTTFont, "J");
  replaceInformatorSymbol("counterplay", qApp->translate("ctString", "with counter-play"), graphicsDir, bExpand, bUseTTFont, "M");
  replaceInformatorSymbol("zugzwang", qApp->translate("ctString", "zugzwang"), graphicsDir, bExpand, bUseTTFont, "U");
  replaceInformatorSymbol("withidea", qApp->translate("ctString", "with the idea"), graphicsDir, bExpand, bUseTTFont, "V");
  replaceInformatorSymbol("onlymove", qApp->translate("ctString", "only move"), graphicsDir, bExpand, bUseTTFont, "W");
  replaceInformatorSymbol("betteris", qApp->translate("ctString", "better is"), graphicsDir, bExpand, bUseTTFont, "Z");
  replaceInformatorSymbol("file", qApp->translate("ctString", "line"), graphicsDir, bExpand, bUseTTFont, "\\");
  replaceInformatorSymbol("diagonal", qApp->translate("ctString", "diagonal"), graphicsDir, bExpand, bUseTTFont, "i");
  replaceInformatorSymbol("centre", qApp->translate("ctString", "centre"), graphicsDir, bExpand, bUseTTFont, "j");
  replaceInformatorSymbol("kside", qApp->translate("ctString", "king's side"), graphicsDir, bExpand, bUseTTFont, "k");
  replaceInformatorSymbol("qside", qApp->translate("ctString", "queen's side"), graphicsDir, bExpand, bUseTTFont, "l");
  replaceInformatorSymbol("weakpt", qApp->translate("ctString", "weak point"), graphicsDir, bExpand, bUseTTFont, "X");
  replaceInformatorSymbol("ending", qApp->translate("ctString", "ending"), graphicsDir, bExpand, bUseTTFont, "m");
  replaceInformatorSymbol("bishoppair", qApp->translate("ctString", "pair of bishops"), graphicsDir, bExpand, bUseTTFont, "o");
  replaceInformatorSymbol("opposbishops", qApp->translate("ctString", "bishops of opposite color"), graphicsDir, bExpand, bUseTTFont, "Y");
  replaceInformatorSymbol("samebishops", qApp->translate("ctString", "bishops of the same color"), graphicsDir, bExpand, bUseTTFont, "q");
  replaceInformatorSymbol("unitedpawns", qApp->translate("ctString", "united pawns"), graphicsDir, bExpand, bUseTTFont, "s");
  replaceInformatorSymbol("seppawns", qApp->translate("ctString", "separated pawns"), graphicsDir, bExpand, bUseTTFont, "t");
  replaceInformatorSymbol("doublepawns", qApp->translate("ctString", "double pawns"), graphicsDir, bExpand, bUseTTFont, "u");
  replaceInformatorSymbol("passedpawn", qApp->translate("ctString", "passed pawn"), graphicsDir, bExpand, bUseTTFont, "v");
  replaceInformatorSymbol("morepawns", qApp->translate("ctString", "advantage in number of pawns"), graphicsDir, bExpand, bUseTTFont, ">");
  replaceInformatorSymbol("timelimit", qApp->translate("ctString", "time"), graphicsDir, bExpand, bUseTTFont, "w");
  replaceInformatorSymbol("without", qApp->translate("ctString", "without"), graphicsDir, bExpand, bUseTTFont, "z");
  replaceInformatorSymbol("with", qApp->translate("ctString", "with"), graphicsDir, bExpand, bUseTTFont, "y");
  replaceInformatorSymbol("etc", qApp->translate("ctString", "etc"), graphicsDir, bExpand, bUseTTFont, "||");
  replaceInformatorSymbol("see", qApp->translate("ctString", "see"), graphicsDir, bExpand, bUseTTFont, "-");

}

/** Processes the ``movecomment'' environments enclosed by `\movecomment{}'.
Replaces the informator symbols ``novelty'',
``remark'' and ``various'' by their HTML equivalents.
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
*/
void ctString::processMoveEnvironments(bool bUseTTFont)
{
  // Some positions for scanning through the string
  int idx, bidx;

  // Process ``movecomment'' environments
  idx = find(QRegExp("\\\\movecomment"));
  while (idx >= 0)
  {
    if (bUseTTFont == true)
    {
      replace(idx, 12, "<SPAN>");
      idx += 6;
    }
    else
    {
      remove(idx, 12);
    }

    // Look for matching closing bracket
    bidx = findMatchingBrace(idx);
    // Cut out opening brace
    remove(idx, 1);
    // Correct index of closing brace
    bidx--;
    if (bidx > 0)
    {
      if (bUseTTFont == true)
      {
        replace(bidx, 1, "</SPAN>");
      }
      else
      {
        remove(bidx, 1);
      }

#ifdef QTVER2X
      // Find ``novelty'' commands with trailing space
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\novelty{}"), 
                        "N ", bUseTTFont, "N ", true);
      // Find ``various'' commands with trailing space
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\various{}"), 
                        "R ", bUseTTFont, "R ", true);
      // Find ``comment'' commands with trailing space
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\comment{}"), 
                        "RR ", bUseTTFont, "RR ", true);
#else
      // Find ``novelty'' commands with trailing space
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\novelty\\{\\}"), 
                        "N ", bUseTTFont, "N ", true);
      // Find ``various'' commands with trailing space
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\various\\{\\}"), 
                        "R ", bUseTTFont, "R ", true);
      // Find ``comment'' commands with trailing space
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\comment\\{\\}"), 
                        "RR ", bUseTTFont, "RR ", true);
#endif
      // Find ``novelty'' commands
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\novelty\\s?"), 
                        "N", bUseTTFont, "N", true);
      // Find ``various'' commands
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\various\\s?"), 
                        "R", bUseTTFont, "R", true);
      // Find ``comment'' commands
      replaceMoveSymbol(idx, &bidx, QRegExp("\\\\comment\\s?"), 
                        "RR", bUseTTFont, "RR", true);

    }
    idx = find(QRegExp("\\\\movecomment"));
  } // End of: Replacing ``movecomment'' environments

}

/** Replaces all occurences of the string \a regName
from the index \a from to the index \a to. 
Replaced is either the \a plainText (if \a bUseTTFont is 
``false'') or a ``</SPAN><SPAN>'' tag containing the \a spanText (if
\a bUseTTFont is ``true'').
@warning Changes the value of the index \a to for the position
of the closing delimiter `}'!
@param from Start index of the current ``move'' environment
@param to End index of the current ``move'' environment
@param regName The string to look for
@param plainText The plain text replacement for the symbol
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
@param spanText Text to replace while using the TrueType font ``DBChess''
@param bEscape Is ``true'' if the \a spanText has to be escaped, ``false''
if not.
*/
void ctString::replaceMoveSymbol(int from,
                                 int *to,
                                 const QRegExp &regExp, 
				 const QString &plainText,
                                 bool bUseTTFont,
                                 const QString &spanText,
                                 bool bEscape)
{
#ifdef QTVER2X
  int matchedLength;
#endif

  // Look for expression
#ifdef QTVER2X
  from = regExp.match(*this, from, &matchedLength);
#else
  from = regExp.search(*this, from);
#endif

  while ((from > 0) && (from < *to))
  {
    // Do we use the TrueType font, i.e. does a <SPAN>
    // env already exist? 
    if (bUseTTFont == true)
    {
      // Yes
      
      // Do we have to ``escape'' the text?
      if (bEscape == true)
      {
        // Yes
        // Replace text with ``escaped'' <SPAN> text 
#ifdef QTVER2X
	replace(from, matchedLength, 
                "</SPAN>" + spanText + "<SPAN>");
        // Correct index for last delimiter
        *to += ((spanText.length()+13) - matchedLength);
        from += ((spanText.length()+13) - matchedLength)+1;
#else
	replace(from, regExp.matchedLength(),
                "</SPAN>" + spanText + "<SPAN>");
        // Correct index for last delimiter
        *to += ((spanText.length()+13) - regExp.matchedLength());
        from += ((spanText.length()+13) - regExp.matchedLength())+1;
#endif
      }
      else
      {
        // No
        // Replace text with spanText
#ifdef QTVER2X
        replace(from, matchedLength, spanText);
        // Correct index for last delimiter
        *to += (spanText.length() - matchedLength);
        from += (spanText.length() - matchedLength)+1;
#else
        replace(from, regExp.matchedLength(), spanText);
        // Correct index for last delimiter
        *to += (spanText.length() - regExp.matchedLength());
        from += (spanText.length() - regExp.matchedLength())+1;
#endif
      }

    }
    else
    {
#ifdef QTVER2X
      // Replace text with normal replacement
      replace(from, matchedLength, plainText);
      // Correct index for last delimiter
      *to += (plainText.length() - matchedLength);
      from += (plainText.length() - matchedLength)+1;
#else
      // Replace text with normal replacement
      replace(from, regExp.matchedLength(), plainText);
      // Correct index for last delimiter
      *to += (plainText.length() - regExp.matchedLength());
      from += (plainText.length() - regExp.matchedLength())+1;
#endif
    }

    // Find next occurence
#ifdef QTVER2X
    from = regExp.match(*this, from, &matchedLength);
#else
    from = regExp.search(*this, from);
#endif
  }
 
}

#ifndef QTVER2X
/** Returns the ``matched'' text command that was found by the
regular expression \a rx.
@see processTextEnvironments
@param rx Regular expression that matched a ``text'' command
@return Index of the command (0--3, see defines in \c ctString.cpp)
*/
int ctString::getCapturedCommand(QRegExp &rx)
{ 
  QString matchedString = rx.cap(1);

  if (matchedString == "em")
    return(EMPHASIZE);
  if (matchedString == "it")
    return(ITALIC);
  if (matchedString == "bf")
    return(BOLD);

  return(ROMAN);
}
#endif

/** Replaces the text commands like ``bf'' or ``em''
into their HTML equivalents.
*/
void ctString::processTextEnvironments()
{ 
  // This is where things get a little bit complicated. 
  // We can't simply replace matching brackets 
  // but have to check if we are in ``normal'', ``bold''
  // or ``italic'' mode, because HTML doesn't allow
  // overlapping environments like:
  // <I> <B> </I> <I> </B></I>
  // This needs to be written as:
  // <I> <B> </B></I><B> </B><I><B> </B></I>

  // The trick is to trace the opened environments
  // up to the found text command, e.g. ``{\it '' 
  // by putting them on a ``stack''. 
  // This is done by managing the current text status
  // with a ``switch'' statement, acting like a finite
  // automaton.
  // If a text command is encountered, a ``neutral'' environment 
  // is created first by closing the current one, which is available
  // as the top of the stack (TOS).
  // Then, the new environment is opened and its status is put on
  // the stack.
  // If a closing brace is found, the current status is popped
  // from the stack, its environment is closed and the old
  // environment (TOS) is re-opened again.

  // This way, we always have a nesting level of at most 2 for
  // the text environments in the HTML output...

  // Position of the next ``text'' command
  int idx;
  // Position of the next closing brace
  int bidx;
  // ``Stack'' for the environments
  vector<int> EnvStack;
  // The nearest command from the current position
  int nearestCommand;
  // String for the replacements
  QString sEnvReplace;

  // Regular expressions for the ``text'' environments
#ifdef QTVER2X
  QRegExp rxCommand("{\\\\[iber][tfm]\\s");
  QRegExp rxBrace("}");
#else
  QRegExp rxCommand("\\{\\\\(it|bf|em|rm)\\s");
  QRegExp rxBrace("\\}");
#endif

  // Initialize stack
  EnvStack.push_back(ROMAN);

#ifdef QTVER2X
  idx = find(rxCommand);
#else
  idx = rxCommand.search(*this);
#endif
  bidx = find(rxBrace);
  
  // Process all occurences of ``text'' commands or ``closing braces''
  while ((idx >= 0) || (bidx >= 0))
  {
    // Which is the nearest command?
    if (idx >= 0)
    {
      // Is the closing brace closer?
      if ((bidx >= 0) && (bidx < idx))
      {
        // Yes
        idx = bidx;
	nearestCommand = CLOSING_BRACE;
      }
      else
      {
        // No, so get the ``matched'' command
#ifdef QTVER2X
	nearestCommand = ROMAN;
	if (mid(idx+2, 2) == "em")
          nearestCommand = EMPHASIZE;
	if (mid(idx+2, 2) == "it")
          nearestCommand = ITALIC;
	if (mid(idx+2, 2) == "bf")
          nearestCommand = BOLD;
#else
        nearestCommand = getCapturedCommand(rxCommand);
#endif
      }
    }
    else
    {
      // Next command has to be a closing brace
      idx = bidx;
      nearestCommand = CLOSING_BRACE;
    }

    switch (nearestCommand)
    {
      case ITALIC:	  switch (EnvStack.back())
      			  {
			    case ROMAN:		sEnvReplace = "<I>";
			    			break;
			    case ITALIC:	sEnvReplace = "";
			    			break;
			    case BOLD:		sEnvReplace = "</B><I>";
			    			break;
			    case BOLD_ITALIC:	sEnvReplace = "</I></B><I>";
			    			break;
			  } 
			  replace(idx, 5, sEnvReplace);
			  EnvStack.push_back(ITALIC);
      			  break;
      case BOLD:	  switch (EnvStack.back())
      			  {
			    case ROMAN:		sEnvReplace = "<B>";
			    			break;
			    case ITALIC:	sEnvReplace = "</I><B>";
			    			break;
			    case BOLD:		sEnvReplace = "";
			    			break;
			    case BOLD_ITALIC:	sEnvReplace = "</I>";
			    			break;
			  }  
			  replace(idx, 5, sEnvReplace);
			  EnvStack.push_back(BOLD);
      			  break;
      case EMPHASIZE:	  switch (EnvStack.back())
      			  {
			    case ROMAN:		sEnvReplace = "<I>";
			  			EnvStack.push_back(ITALIC);
			    			break;
			    case ITALIC:	sEnvReplace = "</I>";
			  			EnvStack.push_back(ROMAN);
			    			break;
			    case BOLD:		sEnvReplace = "<I>";
			  			EnvStack.push_back(BOLD_ITALIC);
			    			break;
			    case BOLD_ITALIC:	sEnvReplace = "</I>";
			  			EnvStack.push_back(BOLD);
			    			break;
			  } 
			  replace(idx, 5, sEnvReplace);
      			  break;
      case ROMAN:	  switch (EnvStack.back())
      			  {
			    case ROMAN:		sEnvReplace = "";
			    			break;
			    case ITALIC:	sEnvReplace = "</I>";
			    			break;
			    case BOLD:		sEnvReplace = "</B>";
			    			break;
			    case BOLD_ITALIC:	sEnvReplace = "</I></B>";
			    			break;
			  }  
			  replace(idx, 5, sEnvReplace);
			  EnvStack.push_back(ROMAN);
      			  break;
      case CLOSING_BRACE: // Close current environment
			  sEnvReplace = "";
      			  switch (EnvStack.back())
      			  {
			    case ITALIC:	sEnvReplace += "</I>";
			    			break;
			    case BOLD:		sEnvReplace += "</B>";
			    			break;
			    case BOLD_ITALIC:	sEnvReplace += "</I></B>";
			    			break;
			  }
			  // Remove TOS
			  if (EnvStack.size() > 1)
			    EnvStack.pop_back();
			  // Open old environment again
      			  switch (EnvStack.back())
      			  {
			    case ITALIC:	sEnvReplace += "<I>";
			    			break;
			    case BOLD:		sEnvReplace += "<B>";
			    			break;
			    case BOLD_ITALIC:	sEnvReplace += "<B><I>";
			    			break;
			  }  
			  replace(idx, 1, sEnvReplace);
      			  break;
    }

    bidx = find(rxBrace, idx);
#ifdef QTVER2X
    idx = find(rxCommand, idx);
#else
    idx = rxCommand.search(*this, idx);
#endif
  } // End of: Replacing text environments

}

/** Replaces all occurences of the informator symbol
command with the given \a baseName in this string
with the ``Informator Symbol'' and a HTML ``tip'' containing the 
appropriate description.
@param baseName The command to look for
@param description The description for the informator symbol
@param graphicsDir The ``graphics'' directory
@param bExpand Is ``true'' if the Informator symbols should be output
in plain text, ``false'' if the \c ALT parameter should be used.
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
@param ttfReplace Text to replace while using the TrueType font ``DBChess''
*/
void ctString::replaceInformatorSymbol(const QString &baseName, 
				       const QString &description,
				       const QString &graphicsDir,
				       bool bExpand,
                                       bool bUseTTFont,
                                       const QString &ttfReplace)
{

  // Can we use the TrueType font?
  if (bUseTTFont == true)
  {
    // Yes

    // Look for command with trailing space
#ifdef QTVER2X
    replace(QRegExp("\\\\" + baseName + "{}"), 
            "<SPAN>" + ttfReplace + " </SPAN>");
#else
    replace(QRegExp("\\\\" + baseName + "\\{\\}"), 
            "<SPAN>" + ttfReplace + " </SPAN>");
#endif
    // Look for command without trailing space
    replace(QRegExp("\\\\" + baseName + "\\s?"), 
            "<SPAN>" + ttfReplace + "</SPAN>");
  }
  else
  {
    // Don't use the TrueType font
    if (bExpand == false)
    {
      // Look for command with trailing space
#ifdef QTVER2X
      replace(QRegExp("\\\\" + baseName + "{}"), 
              "<IMG SRC=\"" + graphicsDir + "isymb.png\" ALT=\"" +
	      description + "\" TITLE=\"" + description + "\"> ");
#else
      replace(QRegExp("\\\\" + baseName + "\\{\\}"), 
              "<IMG SRC=\"" + graphicsDir + "isymb.png\" ALT=\"" +
	      description + "\" TITLE=\"" + description + "\"> ");
#endif
      // Look for command without trailing space
      replace(QRegExp("\\\\" + baseName + "\\s?"), 
              "<IMG SRC=\"" + graphicsDir + "isymb.png\" ALT=\"" +
	      description + "\" TITLE=\"" + description + "\"> ");
    }
    else
    {
      // Look for command with trailing space
#ifdef QTVER2X
      replace(QRegExp("\\\\" + baseName + "{}"), 
              "(" + description + ") ");
#else
      replace(QRegExp("\\\\" + baseName + "\\{\\}"), 
              "(" + description + ") ");
#endif
      // Look for command without trailing space
      replace(QRegExp("\\\\" + baseName + "\\s?"), 
              "(" + description + ")");
    }
  }
}


/** Replaces all occurences of the symbol with the 
given \a baseName in this string with either the 
\a plainText (if \a bUseTTFont is ``false'') or a
``<SPAN>'' tag containing the \a spanText (if
\a bUseTTFont is ``true'').
@param baseName The command to look for
@param plainText The plain text replacement for the symbol
@param bUseTTFont Is ``true'' if the Informator symbols should be output
using the TrueType font ``DBChess'', ``false'' else.
@param spanText Text to replace while using the TrueType font ``DBChess''
*/
void ctString::replacePlainSymbol(const QString &baseName, 
				  const QString &plainText,
                                  bool bUseTTFont,
                                  const QString &spanText)
{

  // Can we use the TrueType font?
  if (bUseTTFont == true)
  {
    // Yes

    // Look for command with trailing space
#ifdef QTVER2X
    replace(QRegExp("\\\\" + baseName + "{}"), 
            "<SPAN>" + spanText + " </SPAN>");
#else
    replace(QRegExp("\\\\" + baseName + "\\{\\}"), 
            "<SPAN>" + spanText + " </SPAN>");
#endif
    // Look for command without trailing space
    replace(QRegExp("\\\\" + baseName + "\\s?"), 
            "<SPAN>" + spanText + "</SPAN>");
  }
  else
  {
    // Don't use the TrueType font

    // Look for command with trailing space
#ifdef QTVER2X
    replace(QRegExp("\\\\" + baseName + "{}"), 
            plainText + " ");
#else
    replace(QRegExp("\\\\" + baseName + "\\{\\}"), 
            plainText + " ");
#endif
    // Look for command without trailing space
    replace(QRegExp("\\\\" + baseName + "\\s?"), 
            plainText);

  }
}


