/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctTaskList.cpp
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

/*------------------------------------------------------------- Includes */

#include "ctTaskList.h"
#include "ctString.h"
#include "ctPieces.h"
#include "ctGlobal.h"
#include "ctPosition.h"
#include "ctHTMLExport.h"
#include "ctSignalRouter.h"

#include <qfile.h>
#include <qtextstream.h>

/*--------------------------------------------------------- Const values */

/** The number of ``ChessCards'' per page */
const int TASKSPERPAGE = 0x8;

/*------------------------------------------------------------ Functions */

/** Default constructor for the \a ctTaskList class.
*/
ctTaskList::ctTaskList() : 
        QObject(),
        listID(0),
#ifdef DEFENGLISH
        bA4Paper(false),
#else
        bA4Paper(true),
#endif
        iLaTeXDiagramSize(TINY),
        bTwoColumns(true),
        bSingleHTML(false),
        bUseTTFont(true),
        bExpandSymbols(false),
        bOutputSolutions(true),
        iFontSize(0)
{
  ;
}

/** Copy constructor for the \a ctTaskList class.
@param elem Reference to the list of tasks that is copied
*/
ctTaskList::ctTaskList(const ctTaskList &elem) : 
        QObject()
{
  tList = elem.tList;
  sMainTitle = elem.sMainTitle;
  listID = elem.listID;
  bA4Paper = elem.bA4Paper;
  iLaTeXDiagramSize = elem.iLaTeXDiagramSize;
  bTwoColumns = elem.bTwoColumns;
  bSingleHTML = elem.bSingleHTML;
  bUseTTFont = elem.bUseTTFont;
  bExpandSymbols = elem.bExpandSymbols;
  bOutputSolutions = elem.bOutputSolutions;
  iFontSize = elem.iFontSize;
}

/** Overloaded operator ``='' with another \a ctTaskList 
as RHA.
@param elem Reference to the list of tasks that is copied
*/
ctTaskList &ctTaskList::operator= (const ctTaskList &elem)
{
  tList = elem.tList;
  sMainTitle = elem.sMainTitle;
  listID = elem.listID;
  bA4Paper = elem.bA4Paper;
  iLaTeXDiagramSize = elem.iLaTeXDiagramSize;
  bTwoColumns = elem.bTwoColumns;
  bSingleHTML = elem.bSingleHTML;
  bUseTTFont = elem.bUseTTFont;
  bExpandSymbols = elem.bExpandSymbols;
  bOutputSolutions = elem.bOutputSolutions;
  iFontSize = elem.iFontSize;

  return *this;
}

/** Copies the options from another \a ctTaskList.
@param elem Reference to the list of tasks whose options are copied
*/
void ctTaskList::getOptions(const ctTaskList &elem)
{
  bA4Paper = elem.bA4Paper;
  iLaTeXDiagramSize = elem.iLaTeXDiagramSize;
  bTwoColumns = elem.bTwoColumns;
  bSingleHTML = elem.bSingleHTML;
  bUseTTFont = elem.bUseTTFont;
  bExpandSymbols = elem.bExpandSymbols;
  bOutputSolutions = elem.bOutputSolutions;
  iFontSize = elem.iFontSize;
}

/** Returns the current state (``true'' or ``false'') of 
the option ``Use TrueType font'' for the HTML output.
@return ``true'' or ``false''
*/
bool ctTaskList::getUseTTFontEnabled()
{
  return bUseTTFont;
}

/** Returns the current state (``true'' or ``false'') of 
the option ``ISO A4 paper size'' for the LaTeX output,
especially the ChessCards.
@return ``true'' or ``false''
*/
bool ctTaskList::getA4PaperSizeEnabled()
{
  return bA4Paper;
}

/** Writes the current list of tasks into a single HTML file
\c index.html and all solutions into the file \c sol.html.
@param dirString The name of the output directory
@param graphicsDir The ``graphics'' directory
@param indexString The ``YahooIndex'' string for the top of the pages
*/
void ctTaskList::exportSingleHTML(const QString &dirString,
                                  const QString &graphicsDir,
			          const QString &indexString)
{
  // Iterator for tasks
  vector<ctTask>::iterator t_it;
  // Filename
  QString fileName = dirString + "/index.html";
  // The QFile IO-Device
  QFile f(fileName);
  // The text stream for output to the file
  QTextStream t( &f );
  // Counter
  unsigned int i;

  // Write tasks
  if (f.open(IO_WriteOnly))
  {
    // Write header
    if (sMainTitle.length() > 0)
    {
      writeHTMLHeader(t, ctString(sMainTitle, HTML, 
                                  graphicsDir,
                                  bExpandSymbols,
                                  bUseTTFont), bUseTTFont); 
    }
    else
    {
      writeHTMLHeader(t, tr("No title"), bUseTTFont);
    }

    // Write index string
    if (indexString.isEmpty() == false)
    {
      t << indexString << endl << "<P>" << endl;
    }  

    // Write main title as heading
    if (sMainTitle.length() > 0)
    {
      t << "<H1>" << ctString(sMainTitle, HTML, 
                              graphicsDir, 
                              bExpandSymbols,
                              bUseTTFont);
      t << "</H1>" << endl << endl;
    }

    // Write index
    t << "<H2>" << tr("Index") << "</H2>" << endl << endl;

    // Write index entries
    t_it = tList.begin();

    for (i = 1; t_it != tList.end(); t_it++, i++)
    {
      // Write entry 
      t << "<A HREF=\"./index.html#task" << i << "\">";

      if (t_it->Title.length() > 0)
      {
        t << ctString(t_it->Title, HTML, 
                      graphicsDir, 
                      bExpandSymbols,
                      bUseTTFont);
      }
      else
      {
        t << tr("No title");
      }

      t << "</A><BR>" << endl;

    }

    t << endl;

    // Write tasks
    for (i = 1, t_it = tList.begin(); t_it != tList.end(); t_it++, i++)
    {
      // Write anchor
      t << "<A NAME=\"task" << i << "\"></A>" << endl;
      // Write task title
      if (t_it->Title.length() > 0)
      {
        t << "<H2>" << ctString(t_it->Title, HTML, 
	                        graphicsDir, 
                                bExpandSymbols,
                                bUseTTFont);
	t << "</H2>" << endl << endl;
      }
      
      if (t_it->bDiagram == true)
      {
        // Write position
        t_it->Position.exportHTML(t, graphicsDir);
      }
	  
      t << "<P>" << endl;	  
      if (t_it->Task.length() > 0)
      {
        // Write task
        t << ctString(t_it->Task, HTML, 
                      graphicsDir, 
                      bExpandSymbols,
                      bUseTTFont);
        t << endl;
      }  

      // Do we have a solution for this task?
      if ((t_it->Solution.length() > 0) && (bOutputSolutions == true))
      {
        // Yes, so write anchor for it
	t << "<P>\n<A HREF=\"./sol.html#task" << i << "\">";
	t << tr("Show solution") << "</A>" << endl << endl;
      }
    }
    
    // Write trailer
    writeHTMLTrailer(t, graphicsDir, bUseTTFont);

    f.close();

  }
  else
  {
    // Error, file could not be opened
    return;
  }

  // Is the export of solutions switched off?
  if (bOutputSolutions == false)
  {
    // Then return immediately...
    return;
  }

  fileName = dirString + "/sol.html";
  f.setName(fileName);
  t.setDevice( &f );
  if (f.open(IO_WriteOnly))
  {
    // Write header
    if (sMainTitle.length() > 0)
    {
      writeHTMLHeader(t, tr("Solutions for: ") + 
                      ctString(sMainTitle, HTML, 
                               graphicsDir, 
                               bExpandSymbols,
                               bUseTTFont), bUseTTFont);
    }
    else
    {
      writeHTMLHeader(t, tr("Solutions"), bUseTTFont);
    }

    // Write index string
    if (indexString.isEmpty() == false)
    {
      t << indexString << endl << "<P>" << endl;
    }  

    // Write main title as heading
    t << "<H1>";
    if (sMainTitle.length() > 0)
    {
      t << tr("Solutions for: ") << ctString(sMainTitle, HTML, 
                                             graphicsDir, 
                                             bExpandSymbols,
                                             bUseTTFont);
    }
    else
    {
      t << tr("Solutions");
    }
    t << "</H1>" << endl << endl;

    // Write solutions
    for (i = 1, t_it = tList.begin(); t_it != tList.end(); t_it++, i++)
    {
      if (t_it->Solution.length() > 0)
      {
        // Write anchor
        t << "<P>\n<A NAME=\"task" << i << "\"></A>" << endl;
        // Write task title
        if (t_it->Title.length() > 0)
        {
          t << "<H2>" << ctString(t_it->Title, HTML, 
	                          graphicsDir, 
                                  bExpandSymbols,
                                  bUseTTFont);
	  t << "</H2>" << endl << endl;
        }
      
        // Write solution
        t << ctString(t_it->Solution, HTML, 
                      graphicsDir, 
                      bExpandSymbols,
                      bUseTTFont);

        // Write anchor back to the task
	t << "<P>\n<A HREF=\"./index.html#task" << i << "\">";
	t << tr("Back to task") << "</A>" << endl << endl;
      }
    }
    
    // Write trailer
    writeHTMLTrailer(t, graphicsDir, bUseTTFont);

    f.close();

  }

}



/** Writes the current list of tasks into several HTML files
\c taskx.html and all solutions into the files \c solx.html.
Also produces an index file \c index.html in the specified
directory \a dirString.
@param dirString The name of the output directory
@param graphicsDir The ``graphics'' directory
@param indexString The ``YahooIndex'' string for the top of the pages
*/
void ctTaskList::exportMultipleHTML(const QString &dirString,
                                    const QString &graphicsDir,
				    const QString &indexString)
{
  // Iterator for tasks
  vector<ctTask>::iterator t_it;
  // Filename
  QString fileName = dirString + "/index.html";
  // The QFile IO-Device
  QFile f(fileName);
  // The text stream for output to the file
  QTextStream t( &f );
  // Counter
  unsigned int i;

  // Write index file index.html
  if (f.open(IO_WriteOnly))
  {
    // Write header
    if (sMainTitle.length() > 0)
    {
      writeHTMLHeader(t, ctString(sMainTitle, HTML, 
                                  graphicsDir, 
                                  bExpandSymbols,
                                  bUseTTFont) + 
                      ": " + tr("Index"), bUseTTFont);
    }
    else
    {
      writeHTMLHeader(t, tr("Index"), bUseTTFont);
    }

    // Write index string
    if (indexString.isEmpty() == false)
    {
      t << indexString << endl << "<P>" << endl;
    }  

    // Write main title as heading
    t << "<H1>";
    if (sMainTitle.length() > 0)
    {
      t << tr("Index of: ");
      t << ctString(sMainTitle, HTML, 
                    graphicsDir, 
                    bExpandSymbols,
                    bUseTTFont);
    }
    else
    {
      t << tr("Index");
    }
    t << "</H1>" << endl << endl;

    // Write index entries
    t_it = tList.begin();

    for (i = 1; t_it != tList.end(); t_it++, i++)
    {
      // Write entry 
      t << "<A HREF=\"./task" << i << ".html\">";

      if (t_it->Title.length() > 0)
      {
        t << ctString(t_it->Title, HTML, 
                      graphicsDir, 
                      bExpandSymbols,
                      bUseTTFont);
      }
      else
      {
        t << tr("No title");
      }

      t << "</A><BR>" << endl;

    }

    t << endl;

    // Write trailer
    writeHTMLTrailer(t, graphicsDir, bUseTTFont);

    f.close();

  }
  else
  {
    // Error, file could not be opened
    return;
  }

  // String for the task title
  QString taskTitle;
  // Write all tasks and solutions into separate files
  t_it = tList.begin();

  for (i = 1, t_it = tList.begin(); t_it != tList.end(); t_it++, i++)
  {
    // Open file for the task
    fileName = dirString + "/task" + QString::number(i) + ".html";
    f.setName(fileName);
    t.setDevice( &f );
    if (f.open(IO_WriteOnly))
    {
      taskTitle = "";

      // Write main title
      if (sMainTitle.length() > 0)
      {
        taskTitle += ctString(sMainTitle, HTML, 
                              graphicsDir, 
                              bExpandSymbols,
                              bUseTTFont) + ": ";
      }

      if (t_it->Title.length() > 0)
      {
        taskTitle += ctString(t_it->Title, HTML, 
                              graphicsDir, 
                              bExpandSymbols,
                              bUseTTFont);
      }
      else
      {
        taskTitle += tr("No title");
      }

      // Write header
      writeHTMLHeader(t, taskTitle, bUseTTFont);

      // Write index string
      if (indexString.isEmpty() == false)
      {
        t << indexString << endl << "<P>" << endl;
      }	

      // Write anchors to previous, next and index
      if (i > 1)
      {
        // previous task
        t << "<A HREF=\"./task" << (i-1) << ".html\"><IMG SRC=\"" << graphicsDir;
	t << "back_r.png\" ALT=\"";
	t << tr("Previous task") << "\" WIDTH=47 HEIGHT=47 BORDER=0></A>" << endl;
      }	

      // index
      t << "<A HREF=\"./index.html\"><IMG SRC=\"" << graphicsDir;
      t << "idx.png\" ALT=\"";
      t << tr("Index") << "\" WIDTH=97 HEIGHT=47 BORDER=0></A>" << endl;

      if (i < tList.size())
      {
        // next task
        t << "<A HREF=\"./task" << (i+1) << ".html\"><IMG SRC=\"" << graphicsDir;
	t << "fwd_r.png\" ALT=\"";
	t << tr("Next task") << "\" WIDTH=47 HEIGHT=47 BORDER=0></A>" << endl;
      }	

      t << "<HR>\n\n<P>" << endl;


      // Write task title as heading
      if (t_it->Title.length() > 0)
      {
        t << "<H1>" << ctString(t_it->Title, HTML, 
                                graphicsDir, 
                                bExpandSymbols,
                                bUseTTFont);
	t << "</H1>" << endl << endl;
      }

      if (t_it->bDiagram == true)
      {
        // Write position
        t_it->Position.exportHTML(t, graphicsDir);
      }
	  
      t << "<P>" << endl;	  
      if (t_it->Task.length() > 0)
      {
        // Write task
        t << ctString(t_it->Task, HTML, 
                      graphicsDir, 
                      bExpandSymbols,
                      bUseTTFont);
	t << endl;
      }  

      // Do we have a solution for this task?
      if ((t_it->Solution.length() > 0) && (bOutputSolutions == true))
      {
        // Yes, so write anchor for it
	t << "\n<P>\n<A HREF=\"./sol" << i << ".html\">";
	t << tr("Show solution") << "</A>" << endl << endl;
      }

      t << "<HR>" << endl << endl;
      
      // Write anchors to previous, next and index
      if (i > 1)
      {
        // previous task
        t << "<A HREF=\"./task" << (i-1) << ".html\"><IMG SRC=\"" << graphicsDir;
	t << "back_r.png\" ALT=\"";
	t << tr("Previous task") << "\" WIDTH=47 HEIGHT=47 BORDER=0></A>" << endl;
      }	

      // index
      t << "<A HREF=\"./index.html\"><IMG SRC=\"" << graphicsDir;
      t << "idx.png\" ALT=\"";
      t << tr("Index") << "\" WIDTH=97 HEIGHT=47 BORDER=0></A>" << endl;

      if (i < tList.size())
      {
        // next task
        t << "<A HREF=\"./task" << (i+1) << ".html\"><IMG SRC=\"" << graphicsDir;
	t << "fwd_r.png\" ALT=\"";
	t << tr("Next task") << "\" WIDTH=47 HEIGHT=47 BORDER=0></A>" << endl;
      }	

      // Write trailer
      writeHTMLTrailer(t, graphicsDir, bUseTTFont);

      // Close file
      f.close();
    }
    else
    {
      // Error, file could not be opened
      return;
    }

    if ((t_it->Solution.length() > 0) && (bOutputSolutions == true))
    {
      // Open file for the solution
      fileName = dirString + "/sol" + QString::number(i) + ".html";
      f.setName(fileName);
      t.setDevice( &f );
      if (f.open(IO_WriteOnly))
      {
        // Write header
        if (t_it->Title.length() > 0)
        {
          writeHTMLHeader(t, ctString(t_it->Title, HTML, 
	                              graphicsDir, 
                                      bExpandSymbols,
                                      bUseTTFont) + 
	                  ": " + tr("Solution"), bUseTTFont);
        }
	else
	{
          writeHTMLHeader(t, tr("Solution"), bUseTTFont);
	}

        // Write index string
        if (indexString.isEmpty() == false)
        {
          t << indexString << endl << "<P>" << endl;
        }	

        // Write anchor to task
        t << "<A HREF=\"./task" << i << ".html\">";
	t << tr("Back to task") << "</A>" << endl;
        t << "<HR>\n\n<P>\n<H1>" << tr("Solution");

	if (t_it->Title.length() > 0)
	{
          t << tr(" for: ");
	  t << ctString(t_it->Title, HTML, 
                        graphicsDir, 
                        bExpandSymbols,
                        bUseTTFont);
	}

	t << "</H1>" << endl << endl;

	// Write solution
	t << ctString(t_it->Solution, HTML, 
                      graphicsDir, 
                      bExpandSymbols,
                      bUseTTFont);

	t << "\n<P>\n<HR>" << endl;
        t << "<A HREF=\"./task" << i << ".html\">";
	t << tr("Back to task") << "</A>" << endl;

        // Write trailer
	writeHTMLTrailer(t, graphicsDir, bUseTTFont);

        f.close();
      } // End of: if (f.open())
      else
      {
        // Error, file could not be opened
        return;
      }

    } // End of: if (t_it->Solution.length() > 0)  

  } // End of: for(all tasks)

}  

/** Writes a complete ``ChessCards'' line, consisting of two
framed ``ChessCards'' horizontally centered on the page,
to the text stream \a t in
LaTeX format.
@param t Reference to the QTextStream
@param exportType Specifies the field type that should be exported,
either ``TASK'' or ``SOLUTION''
@param taskIndex The index of the current task (0--(maxTask-1)). This
should be the absolute number, not the task number within a page!
@param exportedTaskIndex The logical index of the current task 
(0--(maxExportedTask)).
@param maxTask The total number of tasks in the list
@param maxExportedTask The total number of tasks that are really
exported
@see slotExportChessCardsFile
*/
void ctTaskList::exportChessCardsLine(QTextStream &t,
                                      int exportType,
                                      int *taskIndex,
                                      int *exportedTaskIndex,
                                      int maxTask,
                                      int maxExportedTask)
{
  // The index of the next task to be exported
  int indexA;
  // The index of the next task after indexA to be exported
  int indexB;

  // Step to the first task that has the 
  // output diagram option enabled
  while ((*taskIndex < maxTask) && 
         (tList[*taskIndex].bDiagram == false))
    (*taskIndex)++;

  if (*taskIndex < maxTask)
    indexA = (*taskIndex)++;
  else
    indexA = -1;

  // Step to the next task that has the 
  // output diagram option enabled
  while ((*taskIndex < maxTask) && 
         (tList[*taskIndex].bDiagram == false))
    (*taskIndex)++;

  if (*taskIndex < maxTask)
    indexB = (*taskIndex)++;
  else
    indexB = -1;

  t << "\\centerline{\\cardbox{";
  // Do we export a TASK or SOLUTION line?
  if (exportType == TASK)
  {
    // A TASK line
    // Is there at least one task left for output?
    if (indexA >= 0)
    {
      // Yes, so export it
      t << QString::number((*exportedTaskIndex)++) << "/";
      t << QString::number(maxExportedTask) << "}{" << endl;
      t << "\\fenboard{";

      tList[indexA].Position.exportFEN(t);
      t << "}}{" << endl;
      tList[indexA].Position.exportMarkerLines(t);
      t << ctString(tList[indexA].Task, LATEX) << endl;
    }
    else
    {
      // No task left for output, so write an empty card
      t << "}{}{";
    }
    t << "}%" << endl;
    t << "\\cardbox{";

    // Is there a second task left for output?
    if (indexB >= 0)
    {
      // Yes, so export it
      t << QString::number((*exportedTaskIndex)++) << "/";
      t << QString::number(maxExportedTask) << "}{" << endl;
      t << "\\fenboard{";

      tList[indexB].Position.exportFEN(t);
      t << "}}{" << endl;
      tList[indexB].Position.exportMarkerLines(t);
      t << ctString(tList[indexB].Task, LATEX) << endl;

    }
    else
    {
      // No task left for output, so write an empty card
      t << "}{}{";
    }
  }
  else
  {
    // Export a SOLUTION line
    // Are there at least two tasks left for output?
    if (indexB >= 0)
    {
      // Yes, so export backside of first card
      t << QString::number(*exportedTaskIndex+1) << "/";
      t << QString::number(maxExportedTask) << "}{}{" << endl;

      t << ctString(tList[indexB].Solution, LATEX) << endl;
    }
    else
    {
      // Only at most one task left for output, so write an empty card
      t << "}{}{";
    }
    t << "}%" << endl;
    t << "\\cardbox{";

    // Is there a second task left for output?
    if (indexA >= 0)
    {
      // Yes, so export it
      t << QString::number(*exportedTaskIndex) << "/";
      t << QString::number(maxExportedTask) << "}{}{" << endl;

      t << ctString(tList[indexA].Solution, LATEX) << endl;
    }
    else
    {
      // No task left for output, so write an empty card
      t << "}{}{";
    }

    // Correct counter for logical task index
    if (indexA >= 0)
      (*exportedTaskIndex)++;
    if (indexB >= 0)
      (*exportedTaskIndex)++;
  }
 
  t << "}}" << endl << endl;

}

/*---------------------------------------------------------------- Slots */

/** Tries to read a ``ChessTask'' file from the opened text stream \a t.
@param t Reference to the QTextStream
*/
void ctTaskList::slotReadFile(QTextStream &t)
{
  // New task
  ctTask *taskNew;
  // The current line of input
  QString input;
  // The text of a read task or solution
  QString readText;
  // Current task no.
  int task;
  // Total number of tasks in the file
  int max;
  // Is ``true'' for files in the old format (version < 2.x)
  bool bOldFormat = false;

  // Read comment line
  input = t.readLine();
  if (input != "$ChessTask File")
  {
    // Error
    return;
  }	

  // Read version number
  input = t.readLine();
  // Check version (< v2.x?)
  if (input.mid(10, 1).toInt() < 2)
  {
    // Yes, so we have to convert from the old format
    bOldFormat = true;
  }

  // Read main title of the tasks
  input = t.readLine();
  if (bOldFormat == false)
    sMainTitle = input;
  else
    sMainTitle = ctString(input, CONVERT);

  // Read number of tasks
  input = t.readLine();
  max = input.toInt();

  for (task = 0; ((task < max) && (t.atEnd() == false)); task++)
  {
    // Get new task
    taskNew = new ctTask;
	
    // Read task title
    input = t.readLine();
    if (bOldFormat == false)
      taskNew->Title = input;
    else
      taskNew->Title = ctString(input, CONVERT);
	
    // Read position
    taskNew->Position.load(t);

    // Read task
    input = t.readLine();
    while (input != "$BEGIN_TASK")
    {
      if (input == "0")
      {
        // Don't print a diagram
        taskNew->bDiagram = false;
      }  
      input = t.readLine();
    }  
    readText = "";  
    input = t.readLine();
    while (input != "$END_TASK")
    {
      readText += input;

      input = t.readLine();
      if (input != "$END_TASK")
      {
        readText += "\n";
      }  
    }  

    if (bOldFormat == false)
      taskNew->Task = readText;
    else
      taskNew->Task = ctString(readText, CONVERT);

    // Read solution
    input = t.readLine();
    while (input != "$BEGIN_SOLUTION")
      input = t.readLine();
    readText = "";  
    input = t.readLine();
    while (input != "$END_SOLUTION")
    {
      readText += input;

      input = t.readLine();
      if (input != "$END_SOLUTION")
      {
        readText += "\n";
      }  
    }  

    if (bOldFormat == false)
      taskNew->Solution = readText;
    else
      taskNew->Solution = ctString(readText, CONVERT);

    // Insert new task
    tList.insert(tList.end(), *taskNew);
  }
  
  listID = 0;
      
}

/** Writes the list of tasks to the text stream \a t in
``ChessTask'' format.
@param t Reference to the QTextStream
*/
void ctTaskList::slotSaveFile(QTextStream &t)
{

  // Write comment line
  t << "$ChessTask File" << endl << "$Version: 2.0" << endl;

  // Write main title of the tasks
  t << sMainTitle << endl;

  // Write number of tasks
  t << tList.size() << endl;

  // Write tasks
  vector<ctTask>::iterator t_it = tList.begin();

  for (; t_it != tList.end(); t_it++)
  {
    // Write task title
    t << t_it->Title << endl;
    // Write position
    t_it->Position.save(t);
    t << endl;

    // Write diagram flag
    if (t_it->bDiagram == false)
    {
      t << "0" << endl;
    }
    else
    {
      t << "1" << endl;
    }

    // Write task
    t << "$BEGIN_TASK" << endl;
    t << t_it->Task << endl;
    t << "$END_TASK" << endl;

    // Write solution
    t << "$BEGIN_SOLUTION" << endl;
    t << t_it->Solution << endl;
    t << "$END_SOLUTION" << endl;

  }
      
}

/** Outputs the list of tasks in LaTeX format to the
QTextStream \a t.
@param t Reference to the QTextStream
*/
void ctTaskList::slotExportLaTeXFile(QTextStream &t)
{
  // Write header
  t << "%% Created by ChessTask 2.0" << endl << endl;

  t << "\\documentclass[";

  // Are we using ISO A4 paper format?
  if (bA4Paper == true)
    t << "a4paper, ";  // Yes
  
  // Output font size
  switch (iFontSize)
  {
    case 0: t << "10pt"; break;
    case 1: t << "11pt"; break;
    case 2: t << "12pt"; break;
    default: t << "10pt"; break;
  }
      
  // Do we have to use the twocolumn option?
  if (bTwoColumns == true)
  {
    // Yes
    t << ", twocolumn";
  }
  t << "]";
  
  t << "{article}" << endl;
  if (ctLanguage == GERMAN)
  {
    t << "\\usepackage{german}" << endl;	     
    t << "\\usepackage[latin1]{inputenc}" << endl;	     
  }

  t << "\\usepackage[ps]{skak}" << endl;
  if (ctLanguage == GERMAN)
  {
    // Define new language for ``skak'' package
    t << "\\newskaklanguage{german}{KDTLSB}" << endl;
    t << "\\skaklanguage[german]" << endl;
  }

  t << "\\def\\littlespace{\\vspace{0.2cm}}" << endl << endl;

  // Set font size for diagrams
  switch (iLaTeXDiagramSize)
  {
    case TINY:   t << "\\tinyboard" << endl;
                 break;
    case SMALL:  t << "\\smallboard" << endl;
                 break;
    case NORMAL: t << "\\normalboard" << endl;
                 break;
    case LARGE:  t << "\\largeboard" << endl;
                 break;
  }	
  
  t << "\\begin{document}" << endl;
  t << "\\frenchspacing" << endl << endl;

  // Write main title
  if (sMainTitle.length() > 0)
  {
    t << "\\section*{";
    t << ctString(sMainTitle, LATEX);
    t << "}" << endl << endl;
  }

  // Write tasks
  vector<ctTask>::iterator t_it = tList.begin();

  for (; t_it != tList.end(); t_it++)
  {
    // Write task title
    if (t_it->Title.length() > 0)
    {
      t << "\\subsection*{";
      t << ctString(t_it->Title, LATEX);
      t << "}" << endl;
    }
    else
    {
      t << "\\littlespace" << endl;
    }

    if (t_it->bDiagram == true)
    {
      // Write position
      t_it->Position.exportLaTeX(t);
      t << endl;

    } // End of: if (bDiagram == true)
	  
    if (t_it->Task.length() > 0)
    {
      // Write task
      t << ctString(t_it->Task, LATEX);
      t << endl << endl;
    }  

  }
       
  // Is the export of solutions switched off?
  if (bOutputSolutions == false)
  {
    // Write trailer...
    t << "\\end{document}" << endl << endl;
    // and then return immediately.
    return;
  }  
  
  // Check if we have to print any solutions at all
  int i = 0;
  for (t_it = tList.begin(); t_it != tList.end(); t_it++)
  {
    // Did we find a solution?
    if (t_it->Solution.length() > 0)
    {
      // Yes, so break loop and print solutions
      i = 1;
      break;
    }
  }
      
  if (i == 1)
  {
    // Start new page for solutions
    t << "\\clearpage" << endl;
    // Restart page numbering
    t << "\\pagenumbering{arabic}" << endl << endl;
    t << tr("\\section*{Solutions");

    if (sMainTitle.length() > 0)
    {
      t << tr(" for: ");
      t << ctString(sMainTitle, LATEX);
      t << "}" << endl;
    }
    else
    {
      t << "}" << endl;
    }
	
    for (t_it = tList.begin(); t_it != tList.end(); t_it++)
    {
      // Do we have to print a solution at all?
      if (t_it->Solution.length() > 0)
      {
        // Write task title
        if (t_it->Title.length() > 0)
        {
          t << "\\subsection*{";
	  t << ctString(t_it->Title, LATEX);
	  t << "}" << endl;
        }
        else
        {
          t << "\\littlespace" << endl;
          t << tr("\\subsection*{No title}") << endl;
        }

        // Write solution
        t << ctString(t_it->Solution, LATEX);
	t << endl;
      }
    }
  } // End of: if (i == 1)  

  // Write trailer
  t << "\\end{document}" << endl << endl;
}

/** Outputs the list of tasks in LaTeX format as ``ChessCards'' to the
QTextStream \a t.
@param t Reference to the QTextStream
*/
void ctTaskList::slotExportChessCardsFile(QTextStream &t)
{
  // Write header
  t << "%% Created by ChessTask 2.0" << endl << endl;

  t << "\\documentclass[";

  // Are we using ISO A4 paper format?
  if (bA4Paper == true)
    t << "a4paper, ";  // Yes
  
  t << "10pt]{article}" << endl;

  if (ctLanguage == GERMAN)
  {
    t << "\\usepackage{german}" << endl;	     
    t << "\\usepackage[latin1]{inputenc}" << endl;	     
  }

  t << "\\usepackage[ps]{skak}" << endl;
  if (ctLanguage == GERMAN)
  {
    // Define new language for ``skak'' package
    t << "\\newskaklanguage{german}{KDTLSB}" << endl;
    t << "\\skaklanguage[german]" << endl;
  }

  // Set diagram size to ``tiny''
  t << "\\tinyboard" << endl << endl;

  // Redefine some margins and width and height of the text...
  t << "%% Redefining some page layout sizes/lengths" << endl;
  t << "\\setlength{\\topmargin}{0mm}" << endl;
  t << "\\setlength{\\headheight}{0mm}" << endl;
  t << "\\setlength{\\headsep}{0mm}" << endl;
  t << "\\setlength{\\textheight}{242mm}" << endl;
  t << "\\setlength{\\textwidth}{5.00in}" << endl;
  t << endl << "%% Left margin" << endl;
  // Odd and even side margins differ for the two supported
  // paper formats...
  if (bA4Paper == true)
  {
    t << "\\setlength{\\oddsidemargin}{0.636in}" << endl;
    t << "\\setlength{\\evensidemargin}{0.636in}" << endl << endl;
  }
  else
  {
    t << "\\setlength{\\oddsidemargin}{0.753in}" << endl;
    t << "\\setlength{\\evensidemargin}{0.753in}" << endl << endl;
  }
  t << "%% Separation for \\fbox (should be (58mm - \\ibwidth)/2)" << endl;
  t << "\\setlength{\\fboxsep}{2mm}" << endl;
  t << "%% Width of the inner text box" << endl;
  t << "\\newdimen\\ibwidth" << endl;
  t << "\\ibwidth=54mm" << endl << endl;

  t << "%% Inner card box" << endl;
  t << "\\def\\innerbox#1{{\\vbox to \\ibwidth{\\vfil%" << endl;
  t << "\\hbox to \\ibwidth{\\hfil#1\\hfil}\\vfil}}}" << endl << endl;

  // Define command for a single ``ChessCard''
  t << "\\def\\cardheader{";
  if (sMainTitle.length() < 36)
  {
    t << sMainTitle;
  }
  else
  {
    t << sMainTitle.mid(0, 35);
  }

  t << "}" << endl;
  t << "\\newcommand\\cardbox[3]{%" << endl;
  t << "\\fbox{\\innerbox{%" << endl;
  t << "\\parbox[t][\\ibwidth]{\\ibwidth}{%" << endl;
  t << "\\hbox to \\ibwidth{\\scriptsize\\cardheader\\hfil#1}" << endl;
  t << "\\vfil\\if#2\\relax" << endl;
  t << "     \\else%" << endl;
  t << "       #2\\hbox to \\ibwidth{\\hfil\\showboard\\hfil}%" << endl;
  t << "     \\fi" << endl;
  t << "#3\\vfil}}}}" << endl << endl;

  t << "\\pagestyle{empty}" << endl << endl;

  // Start the document
  t << "\\begin{document}" << endl;
  t << "\\frenchspacing" << endl << endl;

  // Write tasks
  // Total number of tasks
  int maxTask = tList.size();
  // Total number of tasks that are exported
  int maxExportedTask = 0;
  // Index of the logical task index while exporting the list
  int exportedTaskIndex = 1;
  // Logical index of the task that starts the new page
  int currentPageStartExport = 1;
  // Index of the task that starts the new page
  int currentPageStart = 0;
  // Index of the current task while scanning the list
  int currentTaskIndex = 0;
  // Row counter within page
  int cardRow;

  // Get number of tasks to export
  while (currentTaskIndex < maxTask)
  {
    if (tList[currentTaskIndex].bDiagram == true)
      maxExportedTask++;
    currentTaskIndex++;
  }

  // Do we have to export any tasks at all?
  if (maxExportedTask == 0)
  {
    // No, so tell the user and end the file
    // immediately
    t << tr("No diagrams were marked for being output.") << endl;
    t << "\\end{document}" << endl << endl;
    return;
  }


  // Step to the first task that has the 
  // output diagram option enabled
  while ((currentPageStart < maxTask) && 
         (tList[currentPageStart].bDiagram == false))
    currentPageStart++;

  // Write completely filled pages
  while (currentPageStart < maxTask)
  {
    currentTaskIndex = currentPageStart;
    exportedTaskIndex = currentPageStartExport;
    // Write task page
    for (cardRow = 0; cardRow < 3; cardRow++)
    {
      exportChessCardsLine(t, TASK, 
                           &currentTaskIndex, &exportedTaskIndex,
                           maxTask, maxExportedTask); 
      t << "\\nointerlineskip" << endl;
    }
    exportChessCardsLine(t, TASK, 
                         &currentTaskIndex, &exportedTaskIndex,
                         maxTask, maxExportedTask); 

    // Start solution page
    t << "\\clearpage" << endl;
    // Reset task index counter to start enties
    currentTaskIndex = currentPageStart;
    exportedTaskIndex = currentPageStartExport;
    // Write solution page
    for (cardRow = 0; cardRow < 3; cardRow++)
    {
      exportChessCardsLine(t, SOLUTION, 
                           &currentTaskIndex, &exportedTaskIndex,
                           maxTask, maxExportedTask); 
      t << "\\nointerlineskip" << endl;
    }
    exportChessCardsLine(t, SOLUTION, 
                         &currentTaskIndex, &exportedTaskIndex,
                         maxTask, maxExportedTask); 

    // Step to the next task that has the 
    // output diagram option enabled
    currentPageStart = currentTaskIndex;
    currentPageStartExport = exportedTaskIndex;
    while ((currentPageStart < maxTask) && 
           (tList[currentPageStart].bDiagram == false))
      currentPageStart++;
  }

  // Write trailer
  t << "\\end{document}" << endl << endl;
}

/** Writes the current list of tasks into a single or several HTML file(s).
@param dirString The name of the output directory
@param graphicsDir The ``graphics'' directory
@param indexString The ``YahooIndex'' string for the top of the pages
@see exportSingleHTML, exportMultipleHTML
*/
void ctTaskList::slotExportHTML(const QString &dirString,
                                const QString &graphicsDir,
				const QString &indexString)
{
  if (tList.size() == 0)
  {
    return;
  }  

  if (bSingleHTML == true)
  {
    exportSingleHTML(dirString, graphicsDir, indexString);
  }
  else
  {
    exportMultipleHTML(dirString, graphicsDir, indexString);
  }
}  

/** Reads a FEN file from the text stream \a t and inserts the
found positions in the ``Forsyth Edwards Notation'' into the
list of tasks.
@param t Reference to the QTextStream
*/
void ctTaskList::slotImportFENFile(QTextStream &t)
{
  ctTask *taskNew;
  QString input;

  // Is there something left to read?
  while (t.atEnd() != true)
  {
    // Read line
    input = t.readLine();

    // Is the line empty?
    if (!input.isEmpty())
    {
      // No, so insert next task...

      // Get new task
      taskNew = new ctTask;

      // Get position
      taskNew->importFEN(input);

      // Insert new task
      tList.insert(tList.end(), *taskNew);
    }
  }
  listID = 0;
}

/** Reads an EPD (Extended Position Description)
file from the text stream \a t and inserts the found
positions into the list of tasks.
@param t Reference to the QTextStream
*/
void ctTaskList::slotImportEPDFile(QTextStream &t)
{
  ctTask *taskNew;
  QString input;

  // Is there something left to read?
  while (t.atEnd() != true)
  {
    // Read line
    input = t.readLine();

    // Is the line empty?
    if (!input.isEmpty())
    {
      // No, so insert next task...

      // Get new task
      taskNew = new ctTask;

      // Get position
      taskNew->importEPD(input);

      // Insert new task
      tList.insert(tList.end(), *taskNew);
    }
  }
  listID = 0;
}

/** Deletes all data at once.
*/
void ctTaskList::slotDeleteAll()
{
  if (tList.size() == 0)
  {
    return;
  }
  
  tList.erase(tList.begin(), tList.end());
  listID = 0;
}

/** Inserts a new task into the list.
*/
void ctTaskList::slotInsertTask()
{
  ctTask *taskNew = new ctTask;

  tList.insert(tList.end(), *taskNew);
  
  // Move to the last task in the list
  listID = tList.size() - 1;
}

/** Inserts a new task into the list at the
actual position and copies the diagram.
*/
void ctTaskList::slotCopyTask()
{
  if (tList.size() == 0)
  {
    return;
  }

  // Get new task 
  ctTask *taskNew = new ctTask;
  // Iterator to actual task
  vector<ctTask>::iterator t_it = tList.begin()+listID;

  // Copy diagram 
  taskNew->Position = t_it->Position;

  tList.insert(t_it+1, *taskNew);
  
  // Move to the new task in the list
  listID++;

}

/** Deletes the current task.
*/
void ctTaskList::slotDeleteTask()
{
  if (tList.size() == 0)
  {
    return;
  }  
  
  // Iterator for the list
  vector<ctTask>::iterator t_it = tList.begin();
  
  // Erase element
  tList.erase(t_it + listID);
    
  // Did we erase the last element in the list?
  if (listID == tList.size())
  {
    // Yes, so step one back
    listID--;
  }  
}

/** Moves one step back in the list of tasks.
*/
void ctTaskList::slotTaskBack()
{
  if (listID > 0)
  {
    listID--;
  }  
}

/** Moves one step forward in the list of tasks.
*/
void ctTaskList::slotTaskForward()
{
  if (listID < tList.size()-1)
  {
    listID++;
  }  
}

/** Moves to the first entry in the list of tasks.
*/
void ctTaskList::slotFirstTask()
{
  listID = 0;
}

/** Moves to the last entry in the list of tasks.
*/
void ctTaskList::slotLastTask()
{
  if (tList.size() > 0)
  {
    listID = tList.size() - 1;
  }  
}

/** Returns the current list position.
@return Current position in the list (0 ... (list.size()-1))
*/
unsigned int ctTaskList::slotGetCurrentID()
{
  return(listID);
}

/** Returns the current size of the list.
@return Current size of the list (0 ... list.size())
*/
unsigned int ctTaskList::slotGetSize()
{
  return(tList.size());
}

/** Automatically creates a title for all
existing tasks, starting with the current task.
*/
void ctTaskList::slotGenerateTitles()
{
  if (tList.size() == 0)
  {
    return;
  }
  
  // Iterator for the list
  vector<ctTask>::iterator t_it = tList.begin();

  // Counter
  int i = 1;

  for (t_it += listID; t_it != tList.end(); t_it++, i++)
  {
    t_it->Title = tr("Task") + " " + QString::number(i);
  }
  
}

/** Sets the ``main title'' for the list of tasks.
@param stringNew New main title
*/
void ctTaskList::slotSetMainTitle(const QString &stringNew)
{
  sMainTitle = stringNew;
}

/** Sets the title for the current task.
@param stringNew New title
*/
void ctTaskList::slotSetTitleText(const QString &stringNew)
{
  if (tList.size() > 0)
  {
    tList[listID].Title = stringNew;
  }  
}

/** Sets the ``task'' text for the current task.
@param stringNew New task text
*/
void ctTaskList::slotSetTaskText(const QString &stringNew)
{
  if (tList.size() > 0)
  {
    tList[listID].Task = stringNew;
  }  
}

/** Sets the ``solution'' text for the current task.
@param stringNew New solution text
*/
void ctTaskList::slotSetSolutionText(const QString &stringNew)
{
  if (tList.size() > 0)
  {
    tList[listID].Solution = stringNew;
  }  
}

/** Sets the diagram output status for the current task.
@param stringNew New diagram output status (true/false)
*/
void ctTaskList::slotSetDiagramEnable(bool enable)
{
  if (tList.size() > 0)
  {
    tList[listID].bDiagram = enable;
  }  
}

/** Sets the position for the current task.
@param stringNew New diagram position
*/
void ctTaskList::slotSetPosition(const ctPosition &positionNew)
{
  if (tList.size() > 0)
  {
    tList[listID].Position = positionNew;
  }  
}

/** Returns the ``main title'' for the list of tasks.
@return The main title
*/
const QString &ctTaskList::slotGetMainTitle()
{
  return(sMainTitle);
}

/** Returns the title for the current task.
@return The title
@warning Before calling this function, the user has to ensure
that the list is not empty!
*/
const QString &ctTaskList::slotGetTitleText()
{
  return(tList[listID].Title);
}

/** Returns the ``task'' text for the current task.
@return The task text
@warning Before calling this function, the user has to ensure
that the list is not empty!
*/
const QString &ctTaskList::slotGetTaskText()
{
  return(tList[listID].Task);
}

/** Returns the ``solution'' text for the current task.
@return The solution text
@warning Before calling this function, the user has to ensure
that the list is not empty!
*/
const QString &ctTaskList::slotGetSolutionText()
{
  return(tList[listID].Solution);
}

/** Returns the diagram output status for the current task.
@return The diagram output status (true/false)
@warning Before calling this function, the user has to ensure
that the list is not empty!
*/
bool ctTaskList::slotGetDiagramEnable()
{
  return(tList[listID].bDiagram);
}

/** Returns the position for the current task.
@return stringNew New diagram position
@warning Before calling this function, the user has to ensure
that the list is not empty!
*/
const ctPosition &ctTaskList::slotGetPosition()
{
  return(tList[listID].Position);
}

/** Sets the status (enabled/disabled) for using
the ISO A4 paper size while writing a LaTeX file.
@param enable The new status for the ISO A4 paper size option
*/
void ctTaskList::slotSetA4PaperSizeEnable(bool enable)
{
  bA4Paper = enable;
}

/** Sets the font size used for the LaTeX
diagrams. The index ranges from 0--3 (TINY--LARGE).
@param iSize The index of the new font size
*/
void ctTaskList::slotSetLaTeXDiagramSize(int iSize)
{
  iLaTeXDiagramSize = iSize;
}

/** Sets the status (enabled/disabled) for using
the twocolumn option while writing a LaTeX file.
@param enable The new status for the twocolumn option
*/
void ctTaskList::slotSetTwoColumnsEnable(bool enable)
{
  bTwoColumns = enable;
}

/** Sets the ``single file'' status (enabled/disabled)
for the HTML ouptut.
@param enable The new status for single file HTML output
*/
void ctTaskList::slotSetOneFileEnable(bool enable)
{
  bSingleHTML = enable;
}

/** Sets the status (enabled/disabled) for using
the TrueType font ``DBChess'' during the HTML output.
@param enable The new status for using the TrueType font ``DBChess''
*/
void ctTaskList::slotSetUseTTFontEnable(bool enable)
{
  bUseTTFont = enable;
}

/** Sets the status (enabled/disabled) for the
expansion of informator symbols to plain text during the HTML output.
@param enable The new status for the expansion of informator symbols
*/
void ctTaskList::slotSetExpandSymbolsEnable(bool enable)
{
  bExpandSymbols = enable;
}

/** Sets the status (enabled/disabled) for the
output of solutions for the LaTeX or HTML export.
@param enable The new status for the output of solutions
*/
void ctTaskList::slotSetOutputSolutionsEnable(bool enable)
{
  bOutputSolutions = enable;
}

/** Sets the new LaTeX font size index.
@param index The new font size index
*/
void ctTaskList::slotSetFontSize(int index)
{
  iFontSize = index;
}  
  

