/* ChessTask - A graphical frontend for creating chess tasks in
*              LaTeX or HTML.
* Copyright (C) 2002-2004 by Dirk Baechle (dl9obn@darc.de)
*
* http://chesstask.sourceforge.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the 
*
* Free Software Foundation, Inc.
* 675 Mass Ave
* Cambridge
* MA 02139
* USA
*
*/

/** 
\file ctTaskList.h
\author Dirk Baechle
\date 2004-03-24
\version 2.0
*/

#ifndef CT_TASK_LIST_H
#define CT_TASK_LIST_H

/*------------------------------------------------------------- Includes */

#include <qobject.h>
#include "ctTask.h"

#include <vector>

using namespace std;

/*------------------------------------------- Forward class declarations */

class QString;
class QTextStream;

/*-------------------------------------------------------------- Classes */

/** This class \a ctTaskList manages a list of \a ctTask in a STL
``vector''.
*/
class ctTaskList: public QObject
{
    Q_OBJECT

public:
    ctTaskList();
    ctTaskList(const ctTaskList &elem);
    ctTaskList &operator= (const ctTaskList &elem);

    void getOptions(const ctTaskList &elem);
    bool getUseTTFontEnabled();
    bool getA4PaperSizeEnabled();

public slots:
    void slotReadFile(QTextStream &t);
    void slotSaveFile(QTextStream &t);
    void slotExportLaTeXFile(QTextStream &t);
    void slotExportChessCardsFile(QTextStream &t);
    void slotExportHTML(const QString &dirString, 
                        const QString &graphicsDir,
			const QString &indexString = QString::null);
    void slotImportFENFile(QTextStream &t);
    void slotImportEPDFile(QTextStream &t);

    void slotDeleteAll();

    void slotInsertTask();
    void slotCopyTask();
    void slotDeleteTask();
    void slotTaskBack();
    void slotTaskForward();
    void slotFirstTask();
    void slotLastTask();

    unsigned int slotGetCurrentID();
    unsigned int slotGetSize();
 
    void slotGenerateTitles();
    void slotSetMainTitle(const QString &);
    void slotSetTitleText(const QString &);
    void slotSetTaskText(const QString &);
    void slotSetSolutionText(const QString &);
    void slotSetDiagramEnable(bool);
    void slotSetPosition(const ctPosition &);

    const QString &slotGetMainTitle();
    const QString &slotGetTitleText();
    const QString &slotGetTaskText();
    const QString &slotGetSolutionText();
    bool slotGetDiagramEnable();
    const ctPosition &slotGetPosition();

    void slotSetA4PaperSizeEnable(bool);
    void slotSetLaTeXDiagramSize(int);
    void slotSetTwoColumnsEnable(bool);
    void slotSetOneFileEnable(bool);
    void slotSetUseTTFontEnable(bool);
    void slotSetExpandSymbolsEnable(bool);
    void slotSetOutputSolutionsEnable(bool);
    void slotSetFontSize(int);

protected:

private:
    /** The main title of the list of tasks */
    QString sMainTitle;
    /** The list of current tasks */
    vector<ctTask> tList;
    /** Current list index */
    unsigned int listID;
 
    /** Is ``true'' if the LaTeX output should use ISO A4 paper size,
        ``false'' for US letter format. */
    bool bA4Paper;
    /** The index (0--3) for the font size of the LaTeX diagrams. */
    int iLaTeXDiagramSize;
    /** Is ``true'' if the LaTeX output should use the twocolumn option. */
    bool bTwoColumns;
    /** Is ``true'' if the HTML export should result in one file only,
        and ``false'' for several files (one for each task) */
    bool bSingleHTML;
    /** Is ``true'' if the informator symbols should be output using the
        TrueType font ``DBChess'', ``false'' else. If ``true'' the value
        of \a bExpandSymbols is disregarded! */
    bool bUseTTFont;
    /** Is ``true'' if the informator symbols should be output in plain text.
        If ``false'' a reference to the GIF image \c isymb.gif is made with an
	\c ALT text, containing the symbol description. */
    bool bExpandSymbols;
    /** Is ``false'' if no solutions should be exported
        and ``true'' if solutions for LaTeX or HTML are provided */
    bool bOutputSolutions;
    /** LaTeX font size index (1=10pt, 2=11pt, 3=12pt) */
    int iFontSize;

    void exportSingleHTML(const QString &dirString, 
                          const QString &graphicsDir,
			  const QString &indexString);
    void exportMultipleHTML(const QString &dirString, 
                            const QString &graphicsDir,
			    const QString &indexString);
    void exportChessCardsLine(QTextStream &t,
                              int exportType,
                              int *taskIndex,
                              int *exportedTaskIndex,
                              int maxTask,
                              int maxExportedTask);

};

#endif

